import 'dart:io';

import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_notification/src/models/notification.dart';
import 'package:ocg_notification/src/service/base.dart';

class NotificationFirebaseService extends NotificationService {
  final FirebaseMessaging firebaseMessaging;

  NotificationFirebaseService() : this.firebaseMessaging = FirebaseMessaging();

  @override
  Future<AppResponse<bool>> subscribe() async {
    try {
      bool isCheck = await firebaseMessaging.requestNotificationPermissions();
      if (Platform.isIOS && !(isCheck ?? false)) {
        return AppResponse.named(error: "IOS permission not granted");
      }
      firebaseMessaging.onTokenRefresh.listen(((val) {
        // TODO
      }));
      // Get Token from firebase
      String fcmToken = await firebaseMessaging.getToken();
      // TODO
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> unsubscribe() async {
    try {
      await firebaseMessaging.deleteInstanceID();
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<void> onNotification(NotificationCallback callback) async {
    firebaseMessaging.configure(
      onMessage: (Map<String, dynamic> message) async {
        callback(_convertToNotification(message));
      },
      onLaunch: (Map<String, dynamic> message) async {
        callback(_convertToNotification(message));
      },
      onResume: (Map<String, dynamic> message) async {
        callback(_convertToNotification(message));
      },
    );
  }

  AppNotification _convertToNotification(temp) {
    var json = temp["data"] ?? temp ?? {};
    return AppNotification.named(
        title: json["title"], description: json["description"], json: json);
  }
}
